#!/usr/bin/env python

import math
from torch import nn
from torch import Tensor

######################################################################

def minimal_input_size(w, layer_specs):
    assert w > 0, 'The input is too small'
    if layer_specs == []:
        return w
    else:
        kernel_size, stride = layer_specs[0]
        v = int(math.ceil((w - kernel_size) / stride)) + 1
        v = minimal_input_size(v, layer_specs[1:])
        return (v - 1) * stride + kernel_size

######################################################################

# Dummy test

if __name__ == "__main__":

    layer_specs = [ (17, 5), (5, 4), (3, 2), (3, 2) ]

    layers = []

    for kernel_size, stride in layer_specs:
        layers.append(nn.Conv2d(1, 1, kernel_size, stride))

    for kernel_size, stride in reversed(layer_specs):
        layers.append(nn.ConvTranspose2d(1, 1, kernel_size, stride))

    m = nn.Sequential(*layers)

    h = minimal_input_size(240, layer_specs)
    w = minimal_input_size(320, layer_specs)

    x = Tensor(1, 1, h, w).normal_()

    print(x.size(), m(x).size())

######################################################################
